/*
 *  world.cc
 *
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "harbour.h"
#include <vector>
#include <time.h>
#include "world.h"
#include "tools.h"
#include "savegame.h"
#include <list>
#include <cmath>
#include <algorithm>

using namespace std;

World::World()
{
   Tools t;
   gturn = 1;
   list<string> harbournames;

   harbournames.push_back("staugustine");
   harbournames.push_back("havana");
   harbournames.push_back("puertoprincipe");
   harbournames.push_back("santodomingo");
   harbournames.push_back("portroyale");
   harbournames.push_back("antigua");
   harbournames.push_back("santacatalina");
   harbournames.push_back("curacao");
   harbournames.push_back("grenada");
   harbournames.push_back("trinidad");

   for (list<string>::iterator hn = harbournames.begin(); hn != harbournames.end(); hn++)
      add_harbour(*hn);

   gameplay = readdata(t.data_path("gameplay-parameters.conf"));
};

harbourlist* World::get_harbours()
{
   return &harbours;
}

playerlist* World::get_players()
{
   return &players;
}

nationlist* World::get_nations()
{
   return &nations;
}

// Return distance between two harbours in nautical miles
float World::compute_harbour_distance(Harbour* from, Harbour* to)
{
   navpos_t f = from->get_position();
   navpos_t t = to->get_position();

   return (hypotf((f.longi-t.longi),(f.lati-t.lati))*60);
}

float World::compute_travel_speed(Ship* ship)
{
   // Calculate weather influence
   return ship->get_speed();
}

Nation* World::get_nation(std::string nationality)
{
   for (nationlist::iterator ni = nations.begin(); ni != nations.end(); ni++)
      if (ni->get_nationality() == nationality)
         return &(*ni);

   return &(nations.front());
}

void World::turn()
{
   gturn++;
   
   for (harbourlist::iterator h = harbours.begin(); h != harbours.end(); h++)
      h->turn(this);

   for (playerlist::iterator pi = players.begin(); pi !=players.end(); pi++)
      pi->turn(this);
}


bool World::is_new_week()
{
   bool week;
   if ((gturn % 7) == 0)
      week = true;
   else
      week = false;
   
   return week;
}


void World::set_state(game_state_t gstate)
{
   game_state = gstate;
}

game_state_t World::get_state()
{
   return game_state;
}

bool World::new_goods_produced()
{
   if (is_new_week())
      return true;
   else 
      return false;
}

Harbour* World::get_harbour_by_name(string harbourname)
{
   for (harbourlist::iterator hi = harbours.begin(); hi != harbours.end(); hi++)
      if (hi->get_name() == harbourname)
         return &(*hi);

   return &(harbours.front());
}


Player* World::get_player_by_name(std::string playername)
{
   for (playerlist::iterator pi = players.begin(); pi != players.end(); pi++)
      if (pi->get_name() == playername)
         return &(*pi);

   return &(players.front());
}

int World::get_nr_players()
{
   return players.size();
}

int World::get_nr_harbours()
{
   return harbours.size();
}

int World::get_nr_nations()
{
   return nations.size();
}


void World::add_player(Player player)
{
   players.push_back(player);
}

void World::add_nation(string nationality)
{
   nations.push_back(Nation(nationality, &players));
}

void World::add_harbour(string harbourname)
{
   Tools t;
   string longpath = t.harbour_data_path(harbourname);

   harbours.push_back(Harbour(longpath));
}

int World::get_turn()
{
   return gturn;
}

Player* World::get_player(int index)
{
   return &players[index];
}

void World::create_opponent()
{
   string opp_nation;
   string pname;
   string opp_shiptype = "NONE";
   vector<string> oname;
   vector<string> oshipname;
   
   Tools t;
   
   int r = rand() % 100;

   if (r >= 0 && r < 20)
   {
      opp_nation = "pi";
      oname.push_back("Calico Jack");
      oname.push_back("Blackbeard");
      oname.push_back("Black Bellamy");
      oname.push_back("William Kidd");
      oname.push_back("Henry Morgan");
      oshipname.push_back("Whydah Gally");
      oshipname.push_back("Black Pearl");
      oshipname.push_back("Polly Fool");
      oshipname.push_back("Halifax");
   }
   
   if (r >= 20 && r < 40)
   {
      opp_nation = "en";
      oname.push_back("Grace O'Malley");
      oname.push_back("Mary Read");
      oname.push_back("Francis Drake");
      oname.push_back("William Dampier");
      oshipname.push_back("Neptune");
      oshipname.push_back("Quedah");
      oshipname.push_back("Mocha");
      oshipname.push_back("Norfolk");
   }

   if (r >= 40 && r < 60)
   {
      opp_nation = "du";
      oname.push_back("Piet Hein");
      oname.push_back("Klaas Hemp");
      oname.push_back("Johannes Vermeer");
      oshipname.push_back("Delft");
      oshipname.push_back("Zeeland");
      oshipname.push_back("Erasmus");
      oshipname.push_back("Beuningen");
   }
   
   if (r >= 60 && r < 80)
   {
      opp_nation = "fr";
      oname.push_back("Jean Lafitte");
      oname.push_back("Jean Bart");
      oname.push_back("Triplette");
      oname.push_back("Samuel de Champlain");
      oshipname.push_back("Concorde");
      oshipname.push_back("Lyon");
      oshipname.push_back("Jean D'Arc");
      oshipname.push_back("Marseille");
   }
   
   if (r >= 80 && r <= 100)
   {
      opp_nation = "sp";
      oname.push_back("Pedro Cofresi");
      oname.push_back("Hernando da Silva");
      oname.push_back("Jose Benem");
      oname.push_back("Miguel de Cervantes");
      oshipname.push_back("Esmeralda");
      oshipname.push_back("Aragon");
      oshipname.push_back("Girona");
      oshipname.push_back("Victoria");
   }
   
   r = rand() % 100;

   if (r >= 0 && r < 20)
      opp_shiptype = "SCHOONER";
   
   if (r >= 20 && r < 40)
      opp_shiptype = "BRIG";
   
   if (r >= 40 && r < 60)
      opp_shiptype = "FRIGATE";
   
   if (r >= 60 && r < 80)
      opp_shiptype = "MERCHANT";

   if (r >= 80 && r <= 100)
      opp_shiptype = "GALLEON";

   if (opp_nation == "pi" && opp_shiptype ==  "MERCHANT")
      opp_shiptype = "BRIG";
   
   if (opp_nation == "pi" && opp_shiptype ==  "GALLEON")
      opp_shiptype = "BRIG";

   Player opp(t.rnd_list(oname), opp_nation, "DIFF_AI", get_harbour_by_name("Antigua"), "Gunning", t.random_col());

   opp.set_ship(opp_shiptype, t.rnd_list(oshipname), 100);

   opp.set_ai(true);

   int cannonmax = opp.get_ship()->get_cannon_capacity();

   opp.get_ship()->add_cannons((int)(cannonmax*gameplay.get_float("cannons_on_opposing_ship")), 0, 0);

   int oppgold = 0;

   if (opp_shiptype == "GALLEON" || opp_shiptype == "MERCHANT")
   {
      vector<string> swaglist;
      swaglist.push_back("Sugar");
      swaglist.push_back("Rum");
      swaglist.push_back("Tobacco");
      swaglist.push_back("Ammo");
      swaglist.push_back("Weapons");

      string lootgood = t.rnd_list(swaglist);
      int maxcargo = opp.get_ship()->get_cargo_capacity();
      opp.get_ship()->add_good(maxcargo, lootgood);
   }

   if (opp_shiptype == "GALLEON")
      oppgold = rand() % gameplay.get_int("maximum_loot_gold_galleon");

   if (opp_shiptype == "MERCHANT")
      oppgold = rand() % gameplay.get_int("maximum_loot_gold_merchant");

   if (opp_shiptype == "FRIGATE")
      oppgold = rand() % gameplay.get_int("maximum_loot_gold_frigate");

   if (opp_shiptype == "SCHOONER")
      oppgold = rand() % gameplay.get_int("maximum_loot_gold_schooner");

   if (opp_shiptype == "BRIG")
      oppgold = rand() % gameplay.get_int("maximum_loot_gold_brig");

   opp.add_gold(oppgold);
   opp.get_ship()->add_food(gameplay.get_float("loot_food")); // FIXME, make this variable
   opponents.push_back(opp);
}

Player* World::get_opponent(int index)
{
   return &opponents[index];
}

void World::remove_players()
{
   players.clear();
}

void World::remove_opponent(int index)
{
   opponents.clear();
}


void World::save_game(std::string filename, std::string savegamename)
{
   Savegame save(filename, false);

   save.write_header(get_turn(), savegamename);
   save.write_game(get_players(), get_harbours(), get_nations(), this);
}

void World::load_game(std::string filename, std::string savegamename)
{
   Savegame save(filename, true);

   add_nation("du");
   add_nation("en");
   add_nation("fr");
   add_nation("sp");
   
   if (save.read_header())
   {
      gturn = save.get_turn();

      harbours.clear();
      vector<Harbour> newh = save.get_harbours();
      harbours.insert(harbours.begin(), newh.begin(), newh.end());

      if (!save.read_check("END-H BEG-P"))
         cerr << "Malformed headers while parsing harbours" << endl;
      
      players.clear();
      vector<Player> newp = save.get_players(this);
      players.insert(players.begin(), newp.begin(), newp.end());

      if (!save.read_check("END-P BEG-N"))
         cerr << "Malformed headers while parsing players" << endl;

      nations.clear();
      vector<Nation> newn = save.get_nations(this);
      nations.insert(nations.begin(), newn.begin(), newn.end());

      if (!save.read_check("END-N"))
         cerr << "Malformed headers while parsing nations" << endl;

   }

   set_state(GS_RUNNING);
}

// FIXME, this is ugly, replace with dynamic allocation
vector<savegame_description> World::get_savegame_names()
{
   Tools t;
   vector<savegame_description> sgs;

   Savegame save1(t.savegame_path("save1.fbs"), true);
   Savegame save2(t.savegame_path("save2.fbs"), true);
   Savegame save3(t.savegame_path("save3.fbs"), true);
   Savegame save4(t.savegame_path("save4.fbs"), true);
   Savegame save5(t.savegame_path("save5.fbs"), true);
   Savegame save6(t.savegame_path("save6.fbs"), true);

   save1.read_header();
   save2.read_header();
   save3.read_header();
   save4.read_header();
   save5.read_header();
   save6.read_header();

   savegame_description s1;
   savegame_description s2;
   savegame_description s3;
   savegame_description s4;
   savegame_description s5;
   savegame_description s6;

   s1.filename = "save1.fbs"; s1.name = save1.get_name();
   s2.filename = "save2.fbs"; s2.name = save2.get_name();
   s3.filename = "save3.fbs"; s3.name = save3.get_name();
   s4.filename = "save4.fbs"; s4.name = save4.get_name(); 
   s5.filename = "save5.fbs"; s5.name = save5.get_name();
   s6.filename = "save6.fbs"; s6.name = save6.get_name();

   sgs.push_back(s1);
   sgs.push_back(s2);
   sgs.push_back(s3);
   sgs.push_back(s4);
   sgs.push_back(s5);
   sgs.push_back(s6);

   return sgs;
}









